package build

import (
	"sync"
)

// compile waits for the scan to finish and compiles all functions.
func compile(functions <-chan *Function, errors <-chan error) (map[string]*Function, error) {
	allFunctions := map[string]*Function{}

	for functions != nil || errors != nil {
		select {
		case err, ok := <-errors:
			if !ok {
				errors = nil
				continue
			}

			return nil, err

		case function, ok := <-functions:
			if !ok {
				functions = nil
				continue
			}

			allFunctions[function.Name] = function
		}
	}

	compileFunctions(allFunctions)

	for _, function := range allFunctions {
		if function.Error != nil {
			return nil, function.Error
		}
	}

	return allFunctions, nil
}

// compileFunctions starts a goroutine for each function compilation and waits for completion.
func compileFunctions(functions map[string]*Function) {
	wg := sync.WaitGroup{}

	for _, function := range functions {
		wg.Add(1)

		go func() {
			defer wg.Done()
			function.Compile()
		}()
	}

	wg.Wait()
}
